import shutil
import subprocess
import tempfile
import zipfile
from argparse import ArgumentParser
from io import BytesIO
from pathlib import Path

import requests


def init_parser(subparsers):
    parser: ArgumentParser = subparsers.add_parser(
        'create', help='Create a new jianmu project.')
    parser.add_argument('project_name',
                        nargs='?',
                        type=Path,
                        help='The directory name of the new project.')
    parser.set_defaults(func=__func)


def __func(args):
    if args.project_name is None:
        print(' * The project name is required.')
        exit(0)
    project_dir: Path = args.project_name.absolute()
    project_name = project_dir.name
    if project_dir.exists():
        print(
            ' * The project directory already exists, please use a different name.'
        )
        exit(0)
    print(f' * Creating a new Jianmu project named {project_name}')
    template_url = 'https://ghproxy.com/https://github.com/frederick-wang/jianmu-template/archive/main.zip'
    print(' * Downloading the template...')
    print(
        ' * (This may take a while, and the program will wait at most 30 seconds)')
    try:
        req = requests.get(template_url, timeout=30)
    except requests.exceptions.RequestException as e:
        print(e)
        print(' * Failed to download the template.')
        exit(0)
    print(' * Downloading the template completed.')
    print(' * Extracting the template...')
    with zipfile.ZipFile(BytesIO(req.content)) as template_archive:
        with tempfile.TemporaryDirectory() as tmpdir:
            template_archive.extractall(tmpdir)
            template_dir = Path(tmpdir) / 'jianmu-template-main'
            shutil.copytree(str(template_dir), str(project_dir))
            print(' * Extracting the template completed.')
    NPM_EXECUTABLE = str(shutil.which('npm'))
    if not NPM_EXECUTABLE:
        print(' * NPM is not installed.')
        exit(0)
    print(' * Installing Node.js dependencies...')
    proc = subprocess.run([
        NPM_EXECUTABLE, 'install', '--registry=https://registry.npmmirror.com'
    ],
                          cwd=str(project_dir))
    if proc.returncode:
        print(' *  Install Node.js dependencies failed.')
        exit(0)
    print('\n * Installing Node.js dependencies completed.')
    print(' * Creating the project completed. Enjoy!\n')
    print(
        'You can now start the project in development mode with following commands:\n'
    )
    print(f'cd {project_name}')
    print('jianmu dev')



from argparse import ArgumentParser


def init_parser(subparsers):
    parser: ArgumentParser = subparsers.add_parser(
        'start', help='Run the jianmu application in production mode.')
    parser.set_defaults(func=__func)


def __func(args):
    print(' * The start command has not been implemented yet.')



import argparse

from ..info import version
from . import build
from . import create
from . import dev
from . import start

parser = argparse.ArgumentParser(
    prog='jianmu',
    description=
    'A simple desktop app development framework combining Python, Vue.js, Element Plus and Electron.',
)


def parse():
    init_parser()
    args = parser.parse_args()
    args.func(args)


def init_parser():
    parser.add_argument('-v',
                        '--version',
                        action='version',
                        version=f'%(prog)s {version}')
    parser.set_defaults(func=__func)

    subparsers = parser.add_subparsers()
    create.init_parser(subparsers)
    dev.init_parser(subparsers)
    start.init_parser(subparsers)
    build.init_parser(subparsers)


def __func(args):
    parser.print_help()


__all__ = ['parse']



import os
import shutil
import signal
import subprocess
import sys
from argparse import ArgumentParser
from pathlib import Path

from ..utils import clear_console


def init_parser(subparsers):
    parser: ArgumentParser = subparsers.add_parser(
        'dev', help='Run the jianmu application in development mode.')
    parser.set_defaults(func=__func)


def __func(args):
    CWD_PATH = Path.cwd()
    PYTHON_EXECUTABLE = sys.executable
    NPM_EXECUTABLE = str(shutil.which('npm'))
    if not NPM_EXECUTABLE:
        print(' * NPM is not installed.')
        exit(0)
    print(' * Starting the jianmu application in development mode...')
    print(f' * Current directory: {CWD_PATH}')
    JIANMU_DIR_PATH = Path(__file__).parent.parent.resolve()
    JM_PATH = JIANMU_DIR_PATH / 'jm.py'
    env = {
        **os.environ,
        'PATH': str(CWD_PATH) + os.pathsep + os.environ['PATH'],
        'DEBUGGING': 'true',
    }
    flask_process: subprocess.Popen[bytes]

    def clean():
        if 'flask_process' in locals() and flask_process.poll() is None:
            flask_process.terminate()
            flask_process.wait()
        if 'node_process' in locals() and node_process.poll() is None:
            node_process.terminate()
            node_process.wait()
        print(' * The jianmu application has been stopped.')
        exit(0)

    try:
        flask_process = subprocess.Popen(args=[PYTHON_EXECUTABLE,
                                               str(JM_PATH)],
                                         cwd=str(CWD_PATH),
                                         env=env)
    except Exception as e:
        print(e)
        print(' * Failed to start the jianmu application.')
        exit(0)

    try:
        node_process = subprocess.Popen(args=['node', 'scripts/dev-server.js'],
                                        cwd=str(CWD_PATH),
                                        env=env)
    except Exception as e:
        print(e)
        print(' * Failed to start the jianmu application.')
        flask_process.terminate()
        flask_process.wait()
        exit(0)

    def on_sigint(signum, frame):
        clean()

    signal.signal(signal.SIGINT, on_sigint)

    while True:
        try:
            flask_process.wait(timeout=1)
            node_process.wait(timeout=1)
            break
        except subprocess.TimeoutExpired:
            if flask_process.poll() is not None or node_process.poll(
            ) is not None:
                break
        except KeyboardInterrupt:
            break

    clean()



from argparse import ArgumentParser


def init_parser(subparsers):
    parser: ArgumentParser = subparsers.add_parser(
        'build', help='Build the jianmu application.')
    parser.set_defaults(func=__func)


def __func(args):
    print(' * The build command has not been implemented yet.')



from setuptools import setup
import setuptools

from jianmu.info import version

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name='Jianmu',
    version=version,
    description=
    'A simple desktop app development framework combining Python, Vue.js, Element Plus and Electron.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Zhaoji Wang',
    author_email='hwoam@outlook.com',
    url='https://github.com/frederick-wang/jianmu',
    project_urls={
        "Bug Tracker": "https://github.com/frederick-wang/jianmu/issues",
    },
    classifiers=[
        'Operating System :: OS Independent',
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Console',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
    ],
    install_requires=[
        'Flask',
        'requests',
    ],
    packages=setuptools.find_packages(include=['jianmu', 'jianmu.*']),
    entry_points={
        'console_scripts': ['jianmu=jianmu.cli:parse'],
    },
    python_requires=">=3.6",
)



import sys
from inspect import signature
from pathlib import Path
from typing import Any, Callable, Dict, List, Sequence, Union

from flask import Flask, request

from exceptions import JianmuException

CWD = str(Path.cwd())
if CWD not in sys.path:
    sys.path.insert(0, CWD)

from src import app

server = Flask(__name__)


@server.route('/')
def index():
    return '<h1>Powered by Jianmu Framework</h1>'


def get_info():
    data = {
        'executable': sys.executable,
        'version': sys.version,
        'versionInfo': sys.version_info,
        'executableDir': str(Path(sys.executable).parent.resolve().absolute()),
        'currentDir': str(Path(__file__).parent.resolve().absolute()),
        'platform': sys.platform,
        'cwd': str(Path.cwd().resolve().absolute()),
    }
    return data, '获取程序信息成功'


JSONValue = Union[str, int, float, bool, None, Dict[str, 'JSONValue'],
                  Sequence['JSONValue']]


def respond(error: int, message: str, data: JSONValue) -> Dict[str, Any]:
    return {
        'error': error,
        'message': message,
        'data': data,
    }


def wrapper(func: Callable):

    def view_func():
        try:
            param_num = len(signature(func).parameters)
            json = request.get_json()
            if json is None:
                raise JianmuException(
                    'The Content-Type header is not application/json')
            args: List[Any] = json
            if param_num:
                if not args:
                    raise JianmuException('The argument is empty')
                if param_num != len(args):
                    raise JianmuException(
                        'The number of arguments do not match')
                data = func(*args)
            else:
                data = func()
            if isinstance(data, tuple):
                if not data:
                    return respond(0, '', None)
                if len(data) == 1:
                    return respond(0, '', data[0])
                if len(data) > 2:
                    raise JianmuException(
                        'The number of return values is greater than 2')
                return respond(0, data[1], data[0])
            return respond(0, '', data)
        except Exception as e:
            return respond(1, e.args[0], None) if e.args else respond(
                1, str(e), None)

    return view_func


if __name__ == '__main__':
    for key, val in app.__dict__.items():
        if key[:2] != '__' and callable(val):
            func_name = key
            func = val
            rule = f'/api/{func_name}'
            view_func = wrapper(func)
            print(f' * [add_url_rule: rule={rule}, func={func_name}]')
            server.add_url_rule(rule, func_name, view_func, methods=['POST'])
    server.add_url_rule('/api/info',
                        'info',
                        wrapper(get_info),
                        methods=['POST'])
    server.run(port=19020, host='0.0.0.0', debug=True)
